/*!\file Vertex.c
 * \brief: implementation of the Vertex object
 */

/*Include files: {{{1*/
#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include <string.h>
#include "./Vertex.h"
#include "../EnumDefinitions/EnumDefinitions.h"
#include "../include/macros.h"
#include "../shared/shared.h"
#include "../include/typedefs.h"
#include "../include/macros.h"
#include "../ModelProcessorx/IoModel.h"
/*}}}*/

/*Object constructors and destructor*/
/*FUNCTION Vertex default constructor {{{1*/
Vertex::Vertex(){
	return;
}
/*}}}*/
/*FUNCTION Vertex constructor {{{1*/
Vertex::Vertex(int tria_id, double tria_x, double tria_y, double tria_z, double tria_sigma, int partitionborder){
	this->Init(tria_id, tria_x, tria_y, tria_z, tria_sigma, partitionborder);
}
/*}}}*/
/*FUNCTION Vertex init, used by constructor {{{1*/
void Vertex::Init(int tria_id, double tria_x, double tria_y, double tria_z, double tria_sigma, int partitionborder){

	/*all the initialization has been done by the initializer, just fill in the id: */
	this->id=tria_id;
	this->x=tria_x;
	this->y=tria_y;
	this->z=tria_z;
	this->sigma=tria_sigma;
	this->dof=UNDEF;

	return;
}
/*}}}*/
/*FUNCTION Vertex constructor  from iomodel{{{1*/
Vertex::Vertex(int i, IoModel* iomodel){

	int partitionborder;

	/*is this vertex on a partition border? */
	if(iomodel->my_bordervertices[i])partitionborder=1;
	else partitionborder=0;

	this->Init(i+1, iomodel->x[i],iomodel->y[i],iomodel->z[i],(iomodel->z[i]-iomodel->bed[i])/(iomodel->thickness[i]),partitionborder);

}
/*}}}*/
/*FUNCTION Vertex destructor {{{1*/
Vertex::~Vertex(){
	return;
}
/*}}}*/

/*Object management*/
/*FUNCTION copy {{{1*/
Object* Vertex::copy() {

	return new Vertex(*this); 

}
/*}}}*/
/*FUNCTION DeepEcho{{{1*/

void Vertex::DeepEcho(void){
	this->Echo();
}
/*}}}*/
/*FUNCTION Demarshall {{{1*/
void  Vertex::Demarshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   i;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*this time, no need to get enum type, the pointer directly points to the beginning of the 
	 *object data (thanks to DataSet::Demarshall):*/

	memcpy(&id,marshalled_dataset,sizeof(id));marshalled_dataset+=sizeof(id);
	memcpy(&x,marshalled_dataset,sizeof(x));marshalled_dataset+=sizeof(x);
	memcpy(&y,marshalled_dataset,sizeof(y));marshalled_dataset+=sizeof(y);
	memcpy(&z,marshalled_dataset,sizeof(z));marshalled_dataset+=sizeof(z);
	memcpy(&sigma,marshalled_dataset,sizeof(sigma));marshalled_dataset+=sizeof(sigma);
	memcpy(&dof,marshalled_dataset,sizeof(dof));marshalled_dataset+=sizeof(dof);
	memcpy(&partitionborder,marshalled_dataset,sizeof(partitionborder));marshalled_dataset+=sizeof(partitionborder);
	memcpy(&clone,marshalled_dataset,sizeof(clone));marshalled_dataset+=sizeof(clone);

	/*return: */
	*pmarshalled_dataset=marshalled_dataset;
	return;
}
/*}}}*/
/*FUNCTION Echo{{{1*/

void Vertex::Echo(void){

	printf("Vertex:\n");
	printf("   id: %i\n",id);
	printf("   x: %g\n",x);
	printf("   y: %g\n",y);
	printf("   z: %g\n",z);
	printf("   sigma: %g\n",sigma);
	printf("   dof: %g\n",dof);
	printf("   partitionborder: %g\n",partitionborder);
	printf("   clone: %g\n",clone);

	return;
}
/*}}}*/
/*FUNCTION Enum {{{1*/
int Vertex::Enum(void){

	return VertexEnum();

}
/*}}}*/
/*FUNCTION GetId{{{1*/
int    Vertex::GetId(void){ return id; }
/*}}}*/
/*FUNCTION GetName{{{1*/
char* Vertex::GetName(void){
	return "node";
}
/*}}}*/
/*FUNCTION Marshall {{{1*/
void  Vertex::Marshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   enum_type=0;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*get enum type of Vertex: */
	enum_type=VertexEnum();

	/*marshall enum: */
	memcpy(marshalled_dataset,&enum_type,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);

	/*marshall Vertex data: */
	memcpy(marshalled_dataset,&id,sizeof(id));marshalled_dataset+=sizeof(id);
	memcpy(marshalled_dataset,&x,sizeof(x));marshalled_dataset+=sizeof(x);
	memcpy(marshalled_dataset,&y,sizeof(y));marshalled_dataset+=sizeof(y);
	memcpy(marshalled_dataset,&z,sizeof(z));marshalled_dataset+=sizeof(z);
	memcpy(marshalled_dataset,&sigma,sizeof(sigma));marshalled_dataset+=sizeof(sigma);
	memcpy(marshalled_dataset,&dof,sizeof(dof));marshalled_dataset+=sizeof(dof);
	memcpy(marshalled_dataset,&partitionborder,sizeof(partitionborder));marshalled_dataset+=sizeof(partitionborder);
	memcpy(marshalled_dataset,&clone,sizeof(clone));marshalled_dataset+=sizeof(clone);

	*pmarshalled_dataset=marshalled_dataset;
	return;
}
/*}}}*/
/*FUNCTION MarshallSize {{{1*/
int   Vertex::MarshallSize(){
	
	return sizeof(id)+
		sizeof(x)+
		sizeof(y)+
		sizeof(z)+
		sizeof(sigma)+
		sizeof(dof)+
		sizeof(partitionborder)+
		sizeof(clone)+
		+sizeof(int); //sizeof(int) for enum type
}
/*}}}*/
/*FUNCTION MyRank {{{1*/
int    Vertex::MyRank(void){ 
	extern int my_rank;
	return my_rank; 
}
/*}}}*/
/*FUNCTION SetClone {{{1*/
void  Vertex::SetClone(int* minranks){

	extern int my_rank;

	if (minranks[id-1]==my_rank){
		this->clone=0;
	}
	else{
		/*!there is a cpu with lower rank that has the same node, 
		therefore, I am a clone*/
		this->clone=1; 	
	}

}
/*}}}*/
/*FUNCTION UpdateFromDakota {{{1*/
void  Vertex::UpdateFromDakota(void* vinputs){

	ISSMERROR("not supported yet!");
	
}
/*}}}*/
/*FUNCTION UpdateFromInputs {{{1*/
void  Vertex::UpdateFromInputs(void* vinputs){
	
	ParameterInputs* inputs=NULL;
	Vertex* vertex=NULL;
	int dof[1]={0};
	double coord[3];

	vertex=this;

	coord[0]=this->x; coord[1]=this->y; coord[2]=this->z;

	/*Recover parameter inputs: */
	inputs=(ParameterInputs*)vinputs;

	/*Update internal data if inputs holds new values: */
	inputs->Recover("x",&coord[0],1,dof,1,(void**)&vertex);
	inputs->Recover("y",&coord[1],1,dof,1,(void**)&vertex);
	inputs->Recover("z",&coord[2],1,dof,1,(void**)&vertex);
	
	ISSMERROR("not supported yet!");
	
}
/*}}}*/
/*FUNCTION UpdateVertexPosition {{{1*/
void  Vertex::UpdatePosition(double* thickness,double* bed){

	/*sigma remains constant. z=bed+sigma*thickness*/
	this->z=bed[dof]+sigma*thickness[this->dof];

}
/*}}}*/
